@@ -2,10 +2,23 @@
 #'
 #' Based on Croston's (1972) method for intermittent demand forecasting, also described in Shenstone and Hyndman (2005). Croston's method involves using simple exponential smoothing (SES) on the non-zero elements of the time series and a separate application of SES to the times between non-zero elements of the time series.
 #'
-#' Note that forecast distributions are not computed as Croston's method has no underlying stochastic model. In a later update, we plan to support distributions via the equivalent stochastic models that underly Croston's method (Shenstone and Hyndman, 2005)
+#' Note that forecast distributions are not computed as Croston's method has no
+#' underlying stochastic model. In a later update, we plan to support distributions via
+#' the equivalent stochastic models that underly Croston's method (Shenstone and
+#' Hyndman, 2005)
+#'
+#' There are two variant methods available which apply multiplicative correction factors
+#' to the forecasts that result from the original Croston's method. For the
+#' Syntetos-Boylan approximation (`type = "sba"`), this factor is \eqn{1 - \alpha / 2},
+#' and for the Shale-Boylan-Johnston method (`type = "sbj"`), this factor is
+#' \eqn{1 - \alpha / (2 - \alpha)}, where \eqn{\alpha} is the smoothing parameter for
+#' the interval SES application.
 #'
 #' @param formula Model specification (see "Specials" section).
 #' @param opt_crit The optimisation criterion used to optimise the parameters.
+#' @param type Which variant of Croston's method to use. Defaults to `"croston"` for
+#' Croston's method, but can also be set to `"sba"` for the Syntetos-Boylan
+#' approximation, and `"sbj"` for the Shale-Boylan-Johnston method.
 #' @param ... Not used.
 #'
 #' @section Specials:
@@ -13,24 +26,26 @@
 #' \subsection{demand}{
 #' The `demand` special specifies parameters for the demand SES application.
 #' \preformatted{
-#' demand(initial = NULL, param = NULL)
+#' demand(initial = NULL, param = NULL, param_range = c(0, 1))
 #' }
 #'
 #' \tabular{ll}{
 #'   `initial`  \tab The initial value for the demand application of SES. \cr
-#'   `param`    \tab The smoothing parameter for the demand application of SES.
+#'   `param`    \tab The smoothing parameter for the demand application of SES. \cr
+#'   `param_range` \tab If `param = NULL`, the range of values over which to search for the smoothing parameter.
 #' }
 #' }
 #'
 #' \subsection{interval}{
 #' The `interval` special specifies parameters for the interval SES application.
 #' \preformatted{
-#' interval(initial = NULL, param = NULL)
+#' interval(initial = NULL, param = NULL, param_range = c(0, 1))
 #' }
 #'
 #' \tabular{ll}{
 #'   `initial`  \tab The initial value for the interval application of SES. \cr
-#'   `param`    \tab The smoothing parameter for the interval application of SES.
+#'   `param`    \tab The smoothing parameter for the interval application of SES. \cr
+#'   `param_range` \tab If `param = NULL`, the range of values over which to search for the smoothing parameter.
 #' }
 #' }
 #'
@@ -65,38 +80,50 @@
 #'   autoplot(sim_poisson)
 #' @importFrom stats model.matrix
 #' @export
-CROSTON <- function(formula, opt_crit = c("mse", "mae"), ...) {
+CROSTON <- function(
+                    formula,
+                    opt_crit = c("mse", "mae"),
+                    type = c("croston", "sba", "sbj"),
+                    ...) {
   opt_crit <- match.arg(opt_crit)
+  type <- match.arg(type)
   croston_model <- new_model_class("croston",
     train = train_croston,
     specials = specials_croston,
     check = all_tsbl_checks
   )
-  new_model_definition(croston_model, !!enquo(formula), opt_crit = opt_crit, ...)
+  new_model_definition(croston_model, !!enquo(formula), opt_crit = opt_crit, type = type, ...)
 }
 
 
 specials_croston <- new_specials(
-  demand = function(initial = NULL, param = NULL) {
+  demand = function(initial = NULL, param = NULL, param_range = c(0, 1)) {
     if (!is.null(initial) && initial < 0) {
       abort("The initial demand for Croston's method must be non-negative")
     }
+    if (param_range[1] > param_range[2]) {
+      rlang::abort("Lower param limits must be less than upper limits")
+    }
 
     as.list(environment())
   },
-  interval = function(initial = NULL, param = NULL, method = c("mean", "naive")) {
+  interval = function(initial = NULL, param = NULL, param_range = c(0, 1), method = c("mean", "naive")) {
     method <- match.arg(method)
 
     if (!is.null(initial) && initial < 1) {
       abort("The initial interval for Croston's method must be greater than (or equal to) 1.")
     }
 
+    if (param_range[1] > param_range[2]) {
+      rlang::abort("Lower param limits must be less than upper limits")
+    }
+
     as.list(environment())
   },
   .required_specials = c("demand", "interval")
 )
 
-train_croston <- function(.data, specials, opt_crit = "mse", ...) {
+train_croston <- function(.data, specials, opt_crit = "mse", type = "croston", ...) {
   if (length(measured_vars(.data)) > 1) {
     abort("Only univariate responses are supported by Croston's method.")
   }
@@ -149,8 +176,9 @@ train_croston <- function(.data, specials, opt_crit = "mse", ...) {
       par = par[par_est], optim_croston, par_est = par_est,
       demand = demand, interval = interval,
       y = y, y_demand = y_demand, y_interval = y_interval,
-      non_zero = non_zero, n = length(y), opt_crit = opt_crit,
-      lower = c(0, 1, 0, 0)[par_est], upper = c(max(y_demand), max(y_interval), 1, 1)[par_est],
+      non_zero = non_zero, n = length(y), type = type, opt_crit = opt_crit,
+      lower = c(0, 1, demand$param_range[1], interval$param_range[1])[par_est],
+      upper = c(max(y_demand), max(y_interval), demand$param_range[2], interval$param_range[2])[par_est],
       method = "L-BFGS-B", control = list(maxit = 2000)
     )$par
   }
@@ -161,7 +189,7 @@ train_croston <- function(.data, specials, opt_crit = "mse", ...) {
   interval$param <- par[4]
 
   fit <- estimate_croston(y_demand, y_interval, demand, interval,
-    non_zero = non_zero, n = length(y)
+    non_zero = non_zero, n = length(y), type = type
   )
 
   structure(
@@ -195,7 +223,7 @@ optim_croston <- function(par, par_est,
   }
 }
 
-estimate_croston <- function(y_demand, y_interval, demand, interval, non_zero, n) {
+estimate_croston <- function(y_demand, y_interval, demand, interval, non_zero, n, type) {
   # Initialise fits
   k <- length(y_demand)
   fit_demand <- numeric(k)
@@ -208,7 +236,14 @@ estimate_croston <- function(y_demand, y_interval, demand, interval, non_zero, n
     fit_demand[i] <- fit_demand[i - 1] + demand$param * (y_demand[i] - fit_demand[i - 1])
     fit_interval[i] <- fit_interval[i - 1] + interval$param * (y_interval[i] - fit_interval[i - 1])
   }
-  ratio <- fit_demand / fit_interval
+  if (type == "sba") {
+    coeff <- 1 - interval$param / 2
+  } else if (type == "sbj") {
+    coeff <- 1 - interval$param / (2 - interval$param)
+  } else {
+    coeff <- 1
+  }
+  ratio <- coeff * fit_demand / fit_interval
 
   # In-sample demand rate
   rep(c(NA_real_, ratio), diff(c(0, non_zero, n)))
@@ -231,7 +266,7 @@ estimate_croston <- function(y_demand, y_interval, demand, interval, non_zero, n
 forecast.croston <- function(object, new_data, specials = NULL, ...) {
   h <- nrow(new_data)
   fc <- rep(object$.fitted[length(object$.fitted)], h)
-  construct_fc(fc, numeric(h), dist_unknown(h))
+  distributional::dist_degenerate(fc)
 }
 
 #' @inherit fitted.ARIMA
